@@ -4,10 +4,11 @@ Utilities for manipulating bokeh models.
 from __future__ import absolute_import, division, unicode_literals
 
 import textwrap
+from contextlib import contextmanager
 
 from bokeh.document import Document
 from bokeh.document.events import ColumnDataChangedEvent
-from bokeh.models import Model, Box
+from bokeh.models import Box, ColumnDataSource, Model
 from bokeh.protocol import Protocol
 
 from .state import state
@@ -61,6 +62,37 @@ def add_to_doc(obj, doc, hold=False):
     if doc._hold is None and hold:
         doc.hold()
 
+@contextmanager
+def hold(doc, policy='combine'):
+    held = doc._hold
+    try:
+        if policy is None:
+            doc.unhold()
+        else:
+            doc.hold(policy)
+        yield
+    finally:
+        if held:
+            doc._hold = held
+        else:
+            doc.unhold()
+
+
+def patch_cds_msg(model, msg):
+    """
+    Required for handling messages containing JSON serialized typed
+    array from the frontend.
+    """
+    for event in msg.get('content', {}).get('events', []):
+        if event.get('kind') != 'ModelChanged' or event.get('attr') != 'data':
+            continue
+        cds = model.select_one({'id': event.get('model').get('id')})
+        if not isinstance(cds, ColumnDataSource):
+            continue
+        for col, values in event.get('new', {}).items():
+            if isinstance(values, dict):
+                event['new'][col] = [v for _, v in sorted(values.items())]
+
 
 _DEFAULT_IGNORED_REPR = frozenset(['children', 'text', 'name', 'toolbar', 'renderers', 'below', 'center', 'left', 'right'])
 
