@@ -113,16 +113,18 @@ class Pipeline(param.Parameterized):
     The Pipeline layout consists of a number of sub-components:
 
     * header:
-      - title: The name of the current stage.
-      - error: A field to display the error state.
-      - network: A network diagram representing the pipeline.
-      - buttons: All navigation buttons and selectors.
-      - prev_button: The button to go to the previous stage.
-      - prev_selector: The selector widget to select between previous
-                       branching stages.
-      - next_button:   The button to go to the previous stage
-      - next_selector: The selector widget to select the next
-                       branching stages.
+
+      * title: The name of the current stage.
+      * error: A field to display the error state.
+      * network: A network diagram representing the pipeline.
+      * buttons: All navigation buttons and selectors.
+      * prev_button: The button to go to the previous stage.
+      * prev_selector: The selector widget to select between
+        previous branching stages.
+      * next_button: The button to go to the previous stage
+      * next_selector: The selector widget to select the next
+        branching stages.
+
     * stage: The contents of the current pipeline stage.
 
     By default any outputs of one stage annotated with the
@@ -140,19 +142,23 @@ class Pipeline(param.Parameterized):
         Whether to automatically advance if the ready parameter is True.""")
 
     debug = param.Boolean(default=False, doc="""
-        Whether to raise errors, useful for debugging while building an application.""")
+        Whether to raise errors, useful for debugging while building
+        an application.""")
 
     inherit_params = param.Boolean(default=True, doc="""
-        Whether parameters should be inherited between pipeline stages""")
+        Whether parameters should be inherited between pipeline
+        stages.""")
 
     next_parameter = param.String(default=None, doc="""
-        Parameter name to watch to switch between different branching stages""")
+        Parameter name to watch to switch between different branching
+        stages""")
 
     ready_parameter = param.String(default=None, doc="""
         Parameter name to watch to check whether a stage is ready.""")
 
     show_header = param.Boolean(default=True, doc="""
-        Whether to show the header with the title, network diagram, and buttons.""")
+        Whether to show the header with the title, network diagram,
+        and buttons.""")
 
     next = param.Action(default=lambda x: x.param.trigger('next'))
 
@@ -504,21 +510,22 @@ class Pipeline(param.Parameterized):
 
         nodes = hv.Nodes(nodes, ['x', 'y', 'Stage'], 'State').opts(
             alpha=0, default_tools=['tap'], hooks=[tap_renderer],
-            hover_alpha=0, selection_alpha=0, nonselection_alpha=0, size=10,
-            backend='bokeh')
+            hover_alpha=0, selection_alpha=0, nonselection_alpha=0,
+            axiswise=True, size=10, backend='bokeh'
+        )
         self._progress_sel.source = nodes
         graph = hv.Graph((edges, nodes)).opts(
             edge_hover_line_color='black', node_color='State', cmap=cmap,
             tools=[], default_tools=['hover'], selection_policy=None,
-            node_hover_fill_color='gray', backend='bokeh')
+            node_hover_fill_color='gray', axiswise=True, backend='bokeh')
         labels = hv.Labels(nodes, ['x', 'y'], 'Stage').opts(
-            yoffset=-.30, default_tools=[], backend='bokeh')
+            yoffset=-.30, default_tools=[], axiswise=True, backend='bokeh'
+        )
         plot = (graph * labels * nodes) if self._linear else (graph * nodes)
         plot.opts(
             xaxis=None, yaxis=None, min_width=400, responsive=True,
-            show_frame=False, height=height, xlim=(-0.25, depth+0.25), ylim=(0, 1),
-            default_tools=['hover'], toolbar=None,
-            backend='bokeh'
+            show_frame=False, height=height, xlim=(-0.25, depth+0.25),
+            ylim=(0, 1), default_tools=['hover'], toolbar=None, backend='bokeh'
         )
         return plot
 
