@@ -502,9 +502,10 @@ def test_empty_tabs_append(document, comm):
 
 def test_tabs_close_tab_in_notebook(document, comm, tabs):
     model = tabs.get_root(document, comm=comm)
+    old_tabs = list(model.tabs)
     _, div2 = tabs
 
-    tabs._comm_change({'tabs': [model.tabs[1].ref['id']]}, model.ref['id'])
+    tabs._comm_change(document, model.ref['id'], 'tabs', old_tabs, [model.tabs[1]])
 
     assert len(tabs.objects) == 1
     assert tabs.objects[0] is div2
