@@ -68,6 +68,8 @@ class Widget(Reactive):
 
     def _update_widget(self, *events):
         for ref, (model, parent) in self._models.items():
+            if ref not in state._views or ref in state._fake_roots:
+                continue
             viewable, root, doc, comm = state._views[ref]
             if comm or state._unblocked(doc):
                 with unlocked():
