@@ -10,9 +10,6 @@ import param
 
 from bokeh.models import Button as _BkButton, Toggle as _BkToggle
 
-from ..config import config
-from ..io.notebook import get_comm_customjs
-from ..io.state import state
 from .base import Widget
 
 
@@ -34,27 +31,10 @@ class Button(_ButtonBase):
 
     _widget_type = _BkButton
 
-    def _link_clicks(self, model, doc, root, comm=None):
-        ref = root.ref['id']
-        if comm is None:
-            model.on_click(partial(self._server_click, doc, ref))
-        elif config.embed:
-            pass
-        else:
-            on_msg = partial(self._comm_change, ref=ref)
-            client_comm = state._comm_manager.get_client_comm(
-                on_msg=on_msg, on_error=partial(self._on_error, ref),
-                on_stdout=partial(self._on_stdout, ref)
-            )
-            customjs = get_comm_customjs(
-                'clicks', client_comm, ref, "value = 1;",
-                self._timeout, self._debounce
-            )
-            model.js_on_click(customjs)
-
     def _get_model(self, doc, root=None, parent=None, comm=None):
         model = super(Button, self)._get_model(doc, root, parent, comm)
-        self._link_clicks(model, doc, root or model, comm)
+        ref = (root or model).ref['id']
+        model.on_click(partial(self._server_click, doc, ref))
         return model
 
     def _server_click(self, doc, ref, event):
@@ -95,6 +75,33 @@ class Button(_ButtonBase):
         from ..links import Callback
         return Callback(self, code={'event:button_click': code}, args=args)
 
+    def jscallback(self, args={}, **callbacks):
+        """
+        Allows defining a JS callback to be triggered when a property
+        changes on the source object. The keyword arguments define the
+        properties that trigger a callback and the JS code that gets
+        executed.
+
+        Arguments
+        ----------
+        args: dict
+          A mapping of objects to make available to the JS callback
+        **callbacks: dict
+          A mapping between properties on the source model and the code
+          to execute when that property changes
+
+        Returns
+        -------
+        callback: Callback
+          The Callback which can be used to disable the callback.
+        """
+        from ..links import Callback
+        for k, v in list(callbacks.items()):
+            if k == 'clicks':
+                k = 'event:button_click'
+            callbacks[k] = self._rename.get(v, v)
+        return Callback(self, code=callbacks, args=args)
+
 
 
 class Toggle(_ButtonBase):
